#!/usr/bin/env python
from django.conf import settings, global_settings as default_settings
from django.core.management import call_command
from os.path import dirname, realpath
import django
import sys
import os
from dynamic_preferences.registries import autodiscover
# Give feedback on used versions
sys.stderr.write('Using Python version {0} from {1}\n'.format(sys.version[:5], sys.executable))
sys.stderr.write('Using Django version {0} from {1}\n'.format(
    django.get_version(),
    os.path.dirname(os.path.abspath(django.__file__)))
)


# Detect location and available modules
module_root = dirname(realpath(__file__))

# Inline settings file
settings.configure(
    DEBUG = False, # will be False anyway by DjangoTestRunner.
    TEMPLATE_DEBUG = False,
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': ':memory:'
        }
    },
    TEMPLATE_CONTEXT_PROCESSORS = default_settings.TEMPLATE_CONTEXT_PROCESSORS + (
        'django.core.context_processors.request',
        'dynamic_preferences.processors.global_preferences',
        'dynamic_preferences.processors.user_preferences',
    ),
    INSTALLED_APPS = (
        'django.contrib.auth',
        'django.contrib.sites',
        'django.contrib.contenttypes',
        'django.contrib.sessions',
        'django.contrib.admin',
        'dynamic_preferences',
    ),
    ROOT_URLCONF = 'dynamic_preferences.urls',
    SITE_ID = 1,    
    STATIC_URL = "/static/",
    TEMPLATE_LOADERS = (
        'django.template.loaders.filesystem.Loader',
        'django.template.loaders.app_directories.Loader',
    ),

    DYNAMIC_PREFERENCES_USE_TEST_PREFERENCES=True,
    LOGGING = {
        'version': 1,
        'disable_existing_loggers': False,
        'formatters': {
            'simple': {
                'format': '%(levelname)s %(message)s'
            },
        },
        'handlers': {
            'console':{
                'level':'DEBUG',
                'class':'logging.StreamHandler',
                'formatter': 'simple'
            },
        },
        'loggers': {
            'django.request':{
                'handlers': ['console'],
                'propagate': True,
                'level': 'DEBUG',
            },
        },
    },
    TESTING=True
)
call_command('syncdb', verbosity=1, interactive=False)
autodiscover()


# ---- app start
verbosity = 2 if '-v' in sys.argv else 1

from django.test.utils import get_runner
TestRunner = get_runner(settings) # DjangoTestSuiteRunner
runner = TestRunner(verbosity=verbosity, interactive=True, failfast=False)
failures = runner.run_tests(['dynamic_preferences'])

if failures:
    sys.exit(bool(failures))


from models import global_preferences as gb, user_preferences as up

def global_preferences(request):
    """
        Pass the values of global preferences to template context.
        You can then access value with `global_preferences.<section>.<name>`
    """
    return {'global_preferences': gb.to_dict()}


def user_preferences(request):
    """
        Pass the values of `request.user` preferences to template context.
        You can then access value with `user_preferences.<section>.<name>`
        If user is not logged in, an empty dictionary will be passed to context 
    """ 

    user = request.user
    if user.is_authenticated():
        return {'user_preferences': up.to_dict(user=user)}

    return {}


from django.core.management.base import BaseCommand, CommandError
from dynamic_preferences.models import GlobalPreferenceModel, UserPreferenceModel, SitePreferenceModel
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from dynamic_preferences.registries import autodiscover, global_preferences_registry, user_preferences_registry, \
    site_preferences_registry


def delete_preferences(queryset):
    """
    Delete preferences objects if they are not present in registry. Return a list of deleted objects
    """
    deleted = []

    # Iterate through preferences. If an error is raised when accessing preference object, just delete it
    for p in queryset:
        try:
            pref = p.preference
        except KeyError:
            p.delete()
            deleted.append(p)

    return deleted


class Command(BaseCommand):
    help = "Find and delete preferences from database if they don't exist in registries. Create preferences that are " \
           "not present in database"

    def handle(self, *args, **options):
        autodiscover()
        deleted = delete_preferences(GlobalPreferenceModel.objects.all())
        print("Deleted {0} global preference models : {1}".format(
            len(deleted),
            ", ".join(['Section: {0} - Name: {1}'.format(p.section, p.name) for p in deleted])
            )
        )

        deleted = delete_preferences(UserPreferenceModel.objects.all())
        print("Deleted {0} user preference models : {1}".format(
            len(deleted),
            ", ".join(['Section: {0} - Name: {1}'.format(p.section, p.name) for p in deleted])
            )
        )

        deleted = delete_preferences(SitePreferenceModel.objects.all())
        print("Deleted {0} site preference models : {1}".format(
            len(deleted),
            ", ".join(['Section: {0} - Name: {1}'.format(p.section, p.name) for p in deleted])
            )
        )

        # Create needed preferences
        # Global
        preferences = global_preferences_registry.preferences()
        for p in preferences:
            p.to_model()

        print('Created/updated default global preferences')

        # User
        preferences = user_preferences_registry.preferences()
        users = User.objects.all()

        for user in users:
            for p in preferences:
                p.to_model(user=user)

        print('Created/updated default preferences for {0} users'.format(len(users)))

        # Site
        preferences = site_preferences_registry.preferences()
        try:
            site = Site.objects.get(pk=1)

        except Site.DoesNotExist:
            print('Cannot create default preference for first site. Please create add at least one site in your '
                  'database.')
            site = None

        if site is not None:
            for p in preferences:
                p.to_model(site=site)

            print('Created/updated default preferences for first site')






from django.test import LiveServerTestCase, TestCase
from dynamic_preferences.preferences import site_preferences_registry, user_preferences_registry, global_preferences_registry, SitePreference, UserPreference
from dynamic_preferences.models import SitePreferenceModel, UserPreferenceModel,PreferenceSite, PreferenceUser, global_preferences, user_preferences

from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.db import IntegrityError
from dynamic_preferences.serializers import *
from django.template import defaultfilters
from dynamic_preferences.registries import autodiscover, clear
from types import *
from dynamic_preferences_registry import *
from dynamic_preferences.forms import global_preference_form_builder, user_preference_form_builder
from django.core.urlresolvers import reverse
from django.core.management import call_command

class TestTutorial(LiveServerTestCase):
    """
    Test everything from the tutorial
    """
    def setUp(self):
        self.henri = User(username="henri", password="test", email="henri@henri.com")
        self.henri.save()

    def test_quickstart(self):

        registration_allowed_preference, created = global_preferences.get_or_create(section="user",
                                                                            name="registration_allowed")
        registration_is_allowed = registration_allowed_preference.value

        self.assertEqual(registration_is_allowed, False)

        registration_allowed_preference.value = True
        registration_allowed_preference.save()

        self.assertEqual(global_preferences.get(section="user", name="registration_allowed").value, True)

        favorite_colour_preference, created = user_preferences.get_or_create(section="misc", name="favourite_colour",
                                                                user=self.henri)
        self.assertEqual(favorite_colour_preference.value, 'Green')

        favorite_colour_preference.value = 'Blue'
        favorite_colour_preference.save()

        self.assertEqual(user_preferences.get(section="misc", name="favourite_colour", user=self.henri).value, 'Blue')

        self.assertEqual(self.henri.preferences.get(section="misc", name="favourite_colour").value, 'Blue')

class TestModels(LiveServerTestCase):
    def test_can_save_and_retrieve_preference_with_section_none(self):
        no_section_pref = global_preferences_registry.get(name="no_section")
        instance = no_section_pref.to_model()
        instance.save()

        self.assertEqual(global_preferences.filter(section=None, name="no_section").count(), 1)

    def test_adding_user_create_default_prefe_rences(self):

        u = User(username="post_create")
        u.save()

        self.assertEqual(u.preferences.count(), len(user_preferences_registry.preferences()))

class TestDynamicPreferences(LiveServerTestCase):

    def setUp(self):

        self.test_user = PreferenceUser(username="test", password="test", email="test@test.com")
        self.test_user.save()

        self.test_site = PreferenceSite(domain="www.test.com", name="test")
        self.test_site.save()

    def test_can_get_preference_value_by_key(self):

        site_pref1 = site_preferences_registry.get("TestSitePref1", "test")
        self.assertEqual(site_pref1.default, TestSitePref1.default)

        user_pref1 = user_preferences_registry.get("TestUserPref1", "test")
        self.assertEqual(user_pref1.default, TestUserPref1.default)

    def test_can_change_site_preference_value(self):

        site_pref1 = site_preferences_registry.get("TestSitePref1", "test")
        site_pref1.value = "new value"

        self.assertEqual(site_preferences_registry.get("TestSitePref1", "test").value, "new value")

        user_pref1 = user_preferences_registry.get("TestUserPref1", "test")
        user_pref1.value = "new value"

        self.assertEqual(user_preferences_registry.get("TestUserPref1","test").value, "new value")

    def test_site_preference_is_saved_to_database(self):

        site_pref1 = site_preferences_registry.get("TestSitePref1", "test")
        site_pref1.to_model(site=self.test_site, value="new site value")

        test_site_pref1 = SitePreferenceModel.objects.get(section="test", name="TestSitePref1", site=self.test_site)
        self.assertEqual(site_pref1, test_site_pref1.preference)
        self.assertEqual(test_site_pref1.section, "test")
        self.assertEqual(test_site_pref1.name, "TestSitePref1")
        self.assertEqual(test_site_pref1.value, "new site value")

    def test_user_preference_is_saved_to_database(self):

        user_pref1 = user_preferences_registry.get("TestUserPref1", "test")
        instance = user_pref1.to_model(user=self.test_user, value="new user value")

        test_user_pref1 = UserPreferenceModel.objects.get(section="test", name="TestUserPref1", user=self.test_user)
        self.assertEqual(user_pref1, test_user_pref1.preference)
        self.assertEqual(test_user_pref1.section, "test")
        self.assertEqual(test_user_pref1.name, "TestUserPref1")
        self.assertEqual(test_user_pref1.value, "new user value")

    def test_site_preference_stay_unique_in_db(self):

        site_pref1 = site_preferences_registry.get("TestSitePref1", "test")
        site_pref1.to_model(site=self.test_site, value="new value")

        duplicate = SitePreferenceModel(section="test", name="TestSitePref1", site=self.test_site)

        with self.assertRaises(IntegrityError):
            duplicate.save()

    def test_user_preference_stay_unique_in_db(self):

        user_pref1 = user_preferences_registry.get("TestUserPref1", "test")
        user_pref1.to_model(user=self.test_user, value="new value")

        duplicate = UserPreferenceModel(section="test", name="TestUserPref1", user=self.test_user)

        with self.assertRaises(IntegrityError):
            duplicate.save()

    def test_preference_value_set_to_default(self):

        pref = user_preferences_registry.get("TestUserPref1", "test")
        pref.to_model(user=self.test_user)

        instance = UserPreferenceModel.objects.get(section="test", name="TestUserPref1", user=self.test_user)
        self.assertEqual(pref.default, instance.value)

        pref = site_preferences_registry.get("TestSitePref1", "test")
        pref.to_model(site=self.test_site)

        instance = SitePreferenceModel.objects.get(section="test", name="TestSitePref1", site=self.test_site)
        self.assertEqual(pref.default, instance.value)


class TestPreferenceObjects(LiveServerTestCase):

    def test_can_get_to_string_notation(self):
        pref = global_preferences_registry.get('user.registration_allowed')

        self.assertEqual(pref.identifier(), 'user.registration_allowed')
        self.assertEqual(pref.identifier("__"), 'user__registration_allowed')

    def test_boolean_field_class_instantiation(self):

        preference = TestBooleanPreference()

        self.assertEqual(preference.field.initial, False)

    def test_char_field_class_instantiation(self):

        preference = TestStringPreference()

        self.assertEqual(preference.field.initial, "hello world!")

    def test_choice_field(self):
        #preference = TestChoicePreference()
        pass
        #self.assertEqual(preference.field.initial, "FR")

class TestRegistry(LiveServerTestCase):

    def test_can_retrieve_preference_using_dotted_notation(self):
        registration_allowed = global_preferences_registry.get(name="registration_allowed", section="user")
        dotted_result = global_preferences_registry.get("user.registration_allowed")
        self.assertEqual(registration_allowed, dotted_result)

    def test_can_register_and_retrieve_preference_with_section_none(self):
        no_section_pref = global_preferences_registry.get(name="no_section")
        self.assertEqual(no_section_pref.section, None)

    def test_can_autodiscover_multiple_times(self):
        autodiscover()
        self.assertEqual(len(global_preferences_registry.preferences()), 8)
        self.assertEqual(len(user_preferences_registry.preferences()), 6)
        autodiscover()
        self.assertEqual(len(global_preferences_registry.preferences()), 8)
        self.assertEqual(len(user_preferences_registry.preferences()), 6)

    def test_can_autodiscover_site_preferences(self):
        clear()
        with self.assertRaises(KeyError):
            site_preferences_registry.preferences(section='test')
        autodiscover(force_reload=True)

        self.assertEqual(len(site_preferences_registry.preferences(section='test')), 4)

    def test_can_autodiscover_user_preferences(self):

        clear()
        with self.assertRaises(KeyError):
            user_preferences_registry.preferences(section='test')

        autodiscover(force_reload=True)

        self.assertEqual(len(user_preferences_registry.preferences(section='test')), 4)


class TestSerializers(LiveServerTestCase):

    def test_boolean_serialization(self):
        s = BooleanSerializer

        self.assertEqual(s.serialize(True), "1")
        self.assertEqual(s.serialize("True"), "1")
        self.assertEqual(s.serialize("Something"), "1")


        self.assertEqual(s.serialize(False), "0")
        self.assertEqual(s.serialize(""), "0")

    def test_boolean_deserialization(self):

        s = BooleanSerializer

        for v in s.true:
            self.assertEqual(s.deserialize(v), True)

        for v in s.false:
            self.assertEqual(s.deserialize(v), False)

        with self.assertRaises(s.exception):
            s.deserialize("I'm a true value")

    def test_int_serialization(self):

        s = IntSerializer

        self.assertEqual(s.serialize(1), "1")
        self.assertEqual(s.serialize(666), "666")
        self.assertEqual(s.serialize(-144), "-144")
        self.assertEqual(s.serialize(0), "0")
        self.assertEqual(s.serialize(123456), "123456")

        with self.assertRaises(s.exception):
            s.serialize("I'm an integer")

    def test_int_deserialization(self):

        s = IntSerializer

        self.assertEqual(s.deserialize("1"), 1)
        self.assertEqual(s.deserialize("125"), 125)
        self.assertEqual(s.deserialize("-144000"), -144000)
        self.assertEqual(s.deserialize("0"), 0)

        with self.assertRaises(s.exception):
            s.deserialize("Trust me, i'm an integer")

    def test_string_serialization(self):

        s = StringSerializer

        self.assertEqual(s.serialize("Bonjour"), "Bonjour")
        self.assertEqual(s.serialize("12"), "12")
        self.assertEqual(s.serialize("I'm a long sentence, but I rock"), "I'm a long sentence, but I rock")

        # check for HTML escaping
        kwargs = {"escape_html": True,}
        self.assertEqual(
            s.serialize("<span>Please, I don't wanna disappear</span>", **kwargs),
            defaultfilters.force_escape("<span>Please, I don't wanna disappear</span>")
        )

        with self.assertRaises(s.exception):
            s.serialize(('I', 'Want', 'To', 'Be', 'A', 'String'))

    def test_string_deserialization(self):

        s = StringSerializer
        self.assertEqual(s.deserialize("Bonjour"), "Bonjour")
        self.assertEqual(s.deserialize("12"), "12")
        self.assertEqual(s.deserialize("I'm a long sentence, but I rock"), "I'm a long sentence, but I rock")

        kwargs = {"escape_html": True,}
        self.assertEqual(
            s.deserialize(s.serialize("<span>Please, I don't wanna disappear</span>", **kwargs)),
            defaultfilters.force_escape("<span>Please, I don't wanna disappear</span>")
        )

        with self.assertRaises(s.exception):
            s.deserialize({"FOR": "THE", "H":0, "R":"DE!!"})

   
class TestViews(LiveServerTestCase):

    def setUp(self):
        self.henri = User(username="henri", password="test", email="henri@henri.com")
        self.henri.set_password('test')
        self.henri.save()

        self.admin = User(username="admin", email="admin@admin.com", is_superuser=True, is_staff=True)
        self.admin.set_password('test')
        self.admin.save()
    def test_can_build_global_preference_form(self):
        # We want to display a form with two global preferences
        # RegistrationAllowed and MaxUsers
        form = global_preference_form_builder(preferences=['user.registration_allowed', "user.max_users"])()

        self.assertEqual(len(form.fields), 2)
        self.assertEqual(form.fields['user.registration_allowed'].initial, False)

    def test_can_build_preference_form_from_sections(self):
        form = global_preference_form_builder(section='test')()

        self.assertEqual(len(form.fields), 3)

    def test_can_build_global_preference_form_from_sections(self):
        form = global_preference_form_builder(section='test')()

        self.assertEqual(len(form.fields), 3)

    def test_can_build_user_preference_form_from_sections(self):
        form = user_preference_form_builder(user=self.admin, section='test')()

        self.assertEqual(len(form.fields), 4)

    def test_global_preference_view_requires_staff_member(self):
        url = reverse("dynamic_preferences.global")
        response = self.client.get(url)
        self.assertIn('body class="login"', response.content)

        self.client.login(username='henri', password="test")
        response = self.client.get(url)
        self.assertIn('body class="login"', response.content)

        self.client.login(username='admin', password="test")
        response = self.client.get(url)

        self.assertEqual(self.admin.is_authenticated(), True)
        self.assertNotIn('body class="login"', response.content)

    def test_global_preference_view_display_form(self):

        url = reverse("dynamic_preferences.global")
        self.client.login(username='admin', password="test")
        response = self.client.get(url)
        self.assertEqual(len(response.context['form'].fields), 8)
        self.assertEqual(response.context['registry'], global_preferences_registry)

    def test_global_preference_filters_by_section(self):
        self.client.login(username='admin', password="test")
        url = reverse("dynamic_preferences.global.section", kwargs={"section": 'user'})
        response = self.client.get(url)
        self.assertEqual(len(response.context['form'].fields), 4)

    def test_preference_are_updated_on_form_submission(self):
        self.client.login(username='admin', password="test")
        url = reverse("dynamic_preferences.global.section", kwargs={"section": 'user'})
        response = self.client.post(url, {'user.max_users': 95, 'user.registration_allowed': True,
                                          'user.favorite_vegetable': "P", "user.items_per_page": 12})
        self.assertEqual(global_preferences.get(section="user", name="max_users").value, 95)
        self.assertEqual(global_preferences.get(section="user", name="registration_allowed").value, True)
        self.assertEqual(global_preferences.get(section="user", name="favorite_vegetable").value, 'P')

    def test_user_preference_form_is_bound_with_current_user(self):
        self.client.login(username='henri', password="test")
        self.assertEqual(user_preferences.get_or_create(user=self.henri, section="misc", name='favourite_colour')[0].value, 'Green')
        self.assertEqual(user_preferences.get_or_create(user=self.henri, section="misc", name='is_zombie')[0].value, True)

        url = reverse("dynamic_preferences.user.section", kwargs={'section': 'misc'})
        response = self.client.post(url, {'misc.favourite_colour': 'Purple', 'misc.is_zombie': False})

        self.assertEqual(self.henri.preferences.get(section="misc", name='favourite_colour').value, 'Purple')
        self.assertEqual(self.henri.preferences.get(section="misc", name='is_zombie').value, False)

    def test_preference_model_manager_to_dict(self):
        call_command('checkpreferences', verbosity=1, interactive=False)
        expected = {u'test': {u'TestGlobal1': u'default value', u'TestGlobal2': False, u'TestGlobal3': False}, None: {u'no_section': False}, u'user': {u'max_users': 100, u'items_per_page': 25, u'registration_allowed': False, u'favorite_vegetable': u'C'}}
        self.assertEqual(global_preferences.to_dict(), expected)

    def test_user_preference_model_manager_to_dict(self):
        call_command('checkpreferences', verbosity=1, interactive=False)
        user = User.objects.get(pk=1)
        expected = {u'misc': {u'favourite_colour': u'Green', u'is_zombie': True}, u'test': {u'SUserStringPref': u'Hello world!', u'SiteBooleanPref': False, u'TestUserPref1': u'default value', u'TestUserPref2': u''}}
        self.assertEqual(user_preferences.to_dict(user=user), expected)


    def test_template_gets_global_preferences_via_template_processor(self):
        call_command('checkpreferences', verbosity=1, interactive=False)
        url = reverse("dynamic_preferences.test.templateview")
        response = self.client.get(url)
        self.assertEqual(response.context['global_preferences'], global_preferences.to_dict())

    def test_template_gets_user_preferences_via_template_processor(self):
        call_command('checkpreferences', verbosity=1, interactive=False)
        user = User.objects.get(pk=1)
        self.client.login(username=user.username, password="test")
        url = reverse("dynamic_preferences.test.templateview")
        response = self.client.get(url)
        to_dict = user_preferences.to_dict(user=user)
        self.assertEqual(response.context['user_preferences'], to_dict)


from dynamic_preferences.preferences import UserPreference, SitePreference, GlobalPreference
from dynamic_preferences.types import *
from dynamic_preferences.registries import register


# Tutorial preferences

@register
class RegistrationAllowed(BooleanPreference, GlobalPreference):
    """
    Are new registrations allowed ?
    """
    section = "user"
    name = "registration_allowed"
    default = False

@register
class MaxUsers(IntPreference, GlobalPreference):
    """
    Are new registrations allowed ?
    """
    section = "user"
    name = "max_users"
    default = 100

@register
class ItemsPerPage(IntPreference, GlobalPreference):
   
    section = "user"
    name = "items_per_page"
    default = 25

@register
class FavoriteVegetable(ChoicePreference, GlobalPreference):

    choices = (
        ("C", "Carrot"),
        ("T", "Tomato. I know, it's not a vegetable"),
        ("P", "Potato")
    )
    section = "user"
    name = "favorite_vegetable"
    default = "C"

@register
class FavouriteColour(UserPreference, StringPreference):
    """
    What's your favourite colour ?
    """
    section = "misc"
    name = "favourite_colour"
    default = "Green"

@register
class IsZombie(BooleanPreference, UserPreference):
    """
    Are you a zombie ?
    """
    section = "misc"
    name = "is_zombie"
    default = True

class BaseTestPref(object):
    section = "test"


# No section pref
@register
class NoSection(BooleanPreference, GlobalPreference):
    name = "no_section"
    default = False



# User preferences
@register
class TestUserPref1(StringPreference, BaseTestPref, UserPreference):
    name = "TestUserPref1"
    default = "default value"

@register
class TestUserPref2(StringPreference, BaseTestPref, UserPreference):
    name = "TestUserPref2"

@register
class UserBooleanPref(BooleanPreference, BaseTestPref, UserPreference):
    name = "SiteBooleanPref"
    default = False

@register
class UserStringPref(StringPreference, BaseTestPref, UserPreference):
    name = "SUserStringPref"
    default = "Hello world!"


# Site Preferences
@register
class TestSitePref1(StringPreference, BaseTestPref, SitePreference):
    name = "TestSitePref1"
    default = "site default value"

@register
class TestSitePref2(StringPreference, BaseTestPref, SitePreference):
    name = "TestSitePref2"

@register
class SiteBooleanPref(BooleanPreference, BaseTestPref, SitePreference):
    name = "SiteBooleanPref"
    default = False

@register
class SiteIntPref(IntPreference, BaseTestPref, SitePreference):
    name = "SiteIntPref"
    default = 2

# Global
@register
class TestGlobal1(StringPreference, BaseTestPref, GlobalPreference):
    name = "TestGlobal1"
    default = "default value"


@register
class TestGlobal2(BooleanPreference, BaseTestPref, GlobalPreference):
    name = "TestGlobal2"
    default = False


@register
class TestGlobal3(BooleanPreference, BaseTestPref, GlobalPreference):
    name = "TestGlobal3"
    default = False







from django.conf.urls import patterns, include, url
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth.decorators import login_required
from dynamic_preferences import views
from registries import user_preferences_registry, global_preferences_registry
from forms import GlobalPreferenceForm

from django.conf import settings

test = False
try:
    test = settings.TESTING
except:
    pass

urlpatterns = patterns('',

    url(r'^global/$', 
        staff_member_required(views.PreferenceFormView.as_view(
            registry=global_preferences_registry, 
            form_class=GlobalPreferenceForm)), 
        name="dynamic_preferences.global"),
    url(r'^global/(?P<section>\w+)$', 
        staff_member_required(views.PreferenceFormView.as_view(
            registry=global_preferences_registry, 
            form_class=GlobalPreferenceForm)), 
        name="dynamic_preferences.global.section"),

    url(r'^user/$', 
        login_required(views.UserPreferenceFormView.as_view()), 
        name="dynamic_preferences.user"),
    url(r'^user/(?P<section>\w+)$', 
        login_required(views.UserPreferenceFormView.as_view()), 
        name="dynamic_preferences.user.section"),
)

if test:

    urlpatterns += patterns('',
        url(r'^test/template$',views.RegularTemplateView.as_view(), name="dynamic_preferences.test.templateview"),      
    )



from dynamic_preferences.types import *
# For testing field instantiation


class TestBooleanPreference(BooleanPreference):
    pass



class TestStringPreference(StringPreference):

    field_attributes = {
        "initial": "hello world!"
    }






from django.conf import settings
from django.utils.importlib import import_module
# import the logging library
import logging

# Get an instance of a logger
logger = logging.getLogger(__name__)

class PreferencesRegistry(dict):
    """
    Registries are special dictionaries that are used by dynamic-preferences to register and access your preferences.
    dynamic-preferences has one registry per Preference type:

    - :py:const:`user_preferences`
    - :py:const:`site_preferences`
    - :py:const:`global_preferences`

    In order to register preferences automatically, you must call :py:func:`autodiscover` in your URLconf.

    """

    #: a name to identify the registry
    name = "preferences_registry"
    def register(self, name, section, preference):
        """
        Store the given preference in the registry. Will also create the preference in database if it does not exist

        This method is called by :py:class:`prefs.BasePreference`.
        You should not have to call it manually.

        :param section: The section name under which the preference should be registered
        :type section: str.
        :param name: The name of the preference
        :type name: str.
        :param preference: a :py:class:`prefs.BasePreference` instance
        """
        try:
            self[section][name] = preference

        except KeyError:
            self[section] = {}
            self[section][name] = preference

    def get(self, name, section=None):
        """
        Returns a previously registered preference

        :param section: The section name under which the preference is registered
        :type section: str.
        :param name: The name of the preference. You can use dotted notation 'section.name' if you want to avoid providing section param
        :type name: str.
        :return: a :py:class:`prefs.BasePreference` instance
        """
        # try dotted notation
        try:
            section, name = name.split('.')
            return self[section][name]
            
        except ValueError:
            pass

        # use standard params
        try:
            return self[section][name]

        except:    
            raise KeyError("No such preference in {0} with section={1} and name={2}".format(
                self.__class__.__name__, section, name))

    def sections(self):
        """
        :return: a list of apps with registered preferences
        :rtype: list
        """

        return self.keys()

    def preferences(self, section=None):
        """
        Return a list of all registered preferences
        or a list of preferences registered for a given section

        :param section: The section name under which the preference is registered
        :type section: str.
        :return: a list of :py:class:`prefs.BasePreference` instances
        """

        if section is None:
            return [self[section][name] for section in self for name in self[section]]
        else:
            return [self[section][name] for name in self[section]]

    def models(self, section=None, **kwargs):
        """
        Return a list of model instances corresponding to registered preferences
        This method calls :py:meth:`preferences.BasePreference.to_model`, see related documentation for more information

        :param section: The section name under which the preferences are registered
        :type section: str.
        :param kwargs: Keyword arguments that will be passed directly to `to_model()`
        :return: a list of :py:class:`models.BasePreferenceModel` instances
        """

        return [preference.to_model(**kwargs) for preference in self.preferences(section)]

    def populate(self, **kwargs):
        """
        Populate database with registered preferences and default values
        """
        raise NotImplementedError

#: The package where autodiscover will try to find preferences to register
preferences_package = "dynamic_preferences_registry"

class GlobalPreferencesRegistry(PreferencesRegistry):
    def populate(self, **kwargs):
        
        return self.models(**kwargs)

class SitePreferencesRegistry(PreferencesRegistry):
    pass

class UserPreferencesRegistry(PreferencesRegistry):
    
    def create_default_preferences(self, user):
        """
            Create default preferences models for a given user
        """
        for preference in self.preferences():
            preference.to_model(user=user)

user_preferences_registry = UserPreferencesRegistry()
site_preferences_registry = SitePreferencesRegistry()
global_preferences_registry = GlobalPreferencesRegistry()

def clear():
    """
    Remove all data from registries
    """
    global_preferences_registry.clear()
    site_preferences_registry.clear()
    user_preferences_registry.clear()

def autodiscover(force_reload=True):
    """
    Populate the registry by iterating through every section declared in :py:const:`settings.INSTALLED_APPS`.

    :param force_reload: if set to `True`, the method will reimport previously imported modules, if any
    :type force_reload: bool.
    """
    print('Dynamic-preferences: autodiscovering preferences...')
    clear()
    prefix = ""

    try:
        test = settings.DYNAMIC_PREFERENCES_USE_TEST_PREFERENCES
        if test:
            # Import test preferences instead of regular ones
            prefix = ".tests"
    except AttributeError, e:
        pass

    for app in settings.INSTALLED_APPS:
        # try to import self.package inside current app
        package = '{0}{1}.{2}'.format(app, prefix, preferences_package)

        try:
            #print('Dynamic-preferences: importing {0}...'.format(package))
            mod = import_module(package)
            if force_reload:
                # mainly used in tests
                reload(mod)

        except ImportError, e:
            pass
            #print('Dynamic-preferences: cannnot import {0}, {1}'.format(package, e))

    global_preferences_registry.populate()

def register(cls):
    instance = cls()

    cls.registry.register(name=cls.name, section=cls.section, preference=instance)
    
    return cls


"""
Preference models, queryset and managers that handle the logic for persisting preferences.
"""

from django.db import models
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.db.models.query import QuerySet
from utils import update
from django.conf import settings
from django.utils.functional import cached_property
from dynamic_preferences.registries import user_preferences_registry, site_preferences_registry, global_preferences_registry

class PreferenceSite(Site):

    class Meta:
        proxy = True
        app_label = 'dynamic_preferences'


class PreferenceUser(User):
    class Meta:
        proxy = True
        app_label = 'dynamic_preferences'


class PreferenceModelManager(models.Manager):

    def to_dict(self, **kwargs):
        """ 
            Return a dict of preference models values with the same structure as registries
            Used to access preferences value within templates
        """ 

        preferences = self.get_queryset().all()
        if kwargs: 
            preferences = preferences.filter(**kwargs)

        d = {}
        for p in preferences:
            try:
                d[p.section][p.name] = p.value
            except KeyError:
                d[p.section] = {}
                d[p.section][p.name] = p.value

        return d

class BasePreferenceModel(models.Model):
    """
    A base model with common logic for all preferences models.
    """

    #: The section under which the preference is declared
    section = models.TextField(max_length=255, db_index=True, blank=True, null=True, default=None)

    #: a name for the preference
    name = models.TextField(max_length=255, db_index=True)

    #: a value, serialized to a string. This field should not be accessed directly, use :py:attr:`BasePreferenceModel.value` instead
    raw_value = models.TextField(null=True, blank=True)

    #: Keep a reference to the whole preference registry.
    #: In order to map the Preference Model Instance to the Preference object.
    registry = None

    objects = PreferenceModelManager()

    class Meta:
        abstract = True
        app_label = 'dynamic_preferences'


    def __init__(self, *args, **kwargs):
        # Check if the model is already saved in DB

        v = kwargs.pop("value", None)

        super(BasePreferenceModel, self).__init__(*args, **kwargs)

        new = self.pk is None

        if new:
            if v is not None:
                self.value = v
            else:
                self.value = self.preference.default



    @cached_property
    def preference(self):
        return self.registry.get(section=self.section, name=self.name)

    def set_value(self, value):
        """
            Save serialized self.value to self.raw_value
        """
        self.raw_value = self.preference.serializer.serialize(value)

    def get_value(self):
        """
            Return deserialized self.raw_value
        """
        return self.preference.serializer.deserialize(self.raw_value)

    value = property(get_value, set_value)

    def __unicode__(self):
        return self.preference.identifier()

class GlobalPreferenceModel(BasePreferenceModel):

    registry = global_preferences_registry

    class Meta:
        unique_together = ('section', 'name')
        app_label = 'dynamic_preferences'

        verbose_name = "global preference"
        verbose_name_plural = "global preferences"


class UserPreferenceModel(BasePreferenceModel):

    user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name="preferences")
    registry = user_preferences_registry

    class Meta:
        unique_together = ('user', 'section', 'name')
        app_label = 'dynamic_preferences'
        verbose_name = "user preference"
        verbose_name_plural = "user preferences"

class SitePreferenceModel(BasePreferenceModel):

    site = models.ForeignKey(Site, related_name="preferences")
    registry = site_preferences_registry

    class Meta:
        unique_together = ('site', 'section', 'name')
        app_label = 'dynamic_preferences'
        verbose_name = "site preference"
        verbose_name_plural = "site preferences"


global_preferences = GlobalPreferenceModel.objects
site_preferences = SitePreferenceModel.objects
user_preferences = UserPreferenceModel.objects

# Create default preferences for new users
# Right now, only works if the model is django.contrib.auth.models.User
# And if settings.CREATE_DEFAULT_PREFERENCES_FOR_NEW_USERS is set to True in settings

create_default_preferencesfor_new_users = getattr(settings, 'CREATE_DEFAULT_PREFERENCES_FOR_NEW_USERS', True)
from django.db.models.signals import post_save

def create_default_preferences(sender, **kwargs): 
    # the object which is saved can be accessed via kwargs 'instance' key.
    obj = kwargs['instance']
    created = kwargs.get("created")
    user_preferences_registry.create_default_preferences(obj)

if create_default_preferencesfor_new_users and settings.AUTH_USER_MODEL == "auth.User":
    from django.contrib.auth.models import User
    post_save.connect(create_default_preferences, sender=User)